#include <cstdlib>
#include "csp-symtable.hpp"
#include "csp-object.hpp"
#include "csp-scope.hpp"

using namespace std;

namespace CSP {

SymTable::SymTable() : scope(0) {
}

// accessors
bool SymTable::lookup(const std::string name,
      ProcessPtr& process) const {
   ProcessDefinitionPtr pdef;
   if (scope) {
      bool found = scope->lookup(name, pdef);
      if (found) process = pdef;
      return found;
   } else {
      return false;
   }
}

// mutators
void SymTable::open() {
   Scope* inner = new Scope(scope);
   scope = inner;
}

void SymTable::close() {
   assert(scope);
   for (std::list<ProcessReferencePtr>::iterator
	 it = unresolved.begin(); it != unresolved.end(); ++it) {
      if (!(*it)->resolve()) {
	 std::cerr << "unable to resolve " <<
	    (*it)->get_name() << std::endl;
	 exit(1);
      }
   }
   Scope* outer = scope->get_outer();
   delete scope;
   scope = outer;
}

bool SymTable::insert(const ProcessDefinitionPtr& process) {
   assert(scope);
   return scope->insert(process);
}
void SymTable::add_unresolved(ProcessReferencePtr pref) {
   assert(pref);
   unresolved.push_back(pref);
}

} // namespace CSP
