#include <iostream>
#include <fstream>
#include <cstdlib>
#include "csp-scanner.hpp"
#include "csp-parser.hpp"
#include "csp-yytname.hpp"
#include "csp-symtable.hpp"

using namespace std;
using namespace CSP;

int main(int argc, char** argv) {
   char* cmdname = *argv++; --argc;
   if (argc > 1) {
      cerr << "Usage: " << cmdname << " [filename]" << endl;
      exit(1);
   }

   SymTable symtab;
   Scanner* scanner;
   ifstream* fin = 0;
   if (argc > 0) {
      char* fname = *argv++; --argc;
      fin = new ifstream(fname);
      string filename(fname);
      if (!*fin) {
	 cerr << cmdname << ": unable to open " << fname <<
	    " for reading" << endl;
	 exit(1);
      }
      scanner = new Scanner(*fin, filename, symtab);
   } else {
      scanner = new Scanner(cin, "stdin", symtab);
   }
   semantic_type yylval;
   int token;
   location loc;
   while ((token = scanner->get_token(yylval, loc)) != 0) {
      cout <<  yytname[token - 255];
      if (yylval) {
	 cout << " \"" << yylval << '"';
	 yylval = ObjectPtr();
      }
      cout << endl;
   }
   delete scanner;
   if (fin) delete fin;
}
